<?php
/**
 * MapDecorate Class
 *
 * Builds a pirate like map.
 *
 * licensed under Attribution-Share Alike 3.0 license
 *
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @license http://creativecommons.org/licenses/by-sa/3.0/
 */
class MapDecorate {
    private $width  = 700;
    private $height = 500;
    private $img    = null;
    private $ll     = '';
    private $text   = '';

    private $layer = array(
        'x'       => array('x.png',50,50),
        'alpha'   => array('alpha.png',700,500),
        'compass' => array('compass.png',200,193),
        'photo'   => array('photo.png',300,300),
        'mask'    => array('mask.png',700,500),
    );

    public $font      = 'AquilineTwo.ttf';
    public $grayscale = false;
    public $apikey    = '';
    public $cachedir  = './cache';


    /**
     * Create the map and output it to the browser
     */
    public function create($ll,$text,$photo=''){
        $this->ll    = $ll;
        $this->text  = $text;
        $this->photo = $photo;

        // use cache
        $cache = $this->cachedir.'/'.md5($ll.$text.$photo.$this->font.$this->grayscale).'.png';
        if(file_exists($cache)){
            header ("Content-type: image/png");
            readfile($cache);
            exit;
        }

        $this->img = @imagecreatetruecolor($this->width, $this->height)
            or die("GD image create failed");
        $color = imagecolorallocate($this->img, 246,240,196);
        imagefill($this->img,0,0,$color);

        $this->placemap();
        $this->compassoverlay();
        $this->placetext();
        $this->alphaoverlay();
        if($photo) $this->placephoto();
        $this->maskoverlay();

        header ("Content-type: image/png");
        @imagepng($this->img,$cache);
        imagepng($this->img);
        imagedestroy($this->img);
    }

    private function placephoto(){
        $data = $this->fetchurl($this->photo);

        $photo = imagecreatefromstring($data);
        if(!$photo) die("failed to create image from photo");

        // bounding box
        $bw = 250;
        $bh = 250;

        // current size
        $w = imagesx($photo);
        $h = imagesy($photo);

        // resize ratio
        $ratio = $this->getResizeRatio($w,$h,$bw,$bh);
        $nw = floor($w * $ratio);
        $nh = floor($h * $ratio);

        // resize image and add border
        $newimg = imagecreatetruecolor ($nw+16, $nh+16);
        $color = imagecolorallocate($newimg, 255,255,255);
        imagefill($newimg,0,0,$color);
        imagecopyresampled($newimg, $photo, 8, 8, 0, 0, $nw, $nh, $w, $h);
        imagedestroy($photo);

        // to grayscale
        if($this->grayscale) imagefilter($newimg,IMG_FILTER_GRAYSCALE);

        // add "old" layer
        $xim = imagecreatefrompng($this->layer['photo'][0]);
        if(!$xim) die('Failed to load layer photo');
        imageSaveAlpha($xim, true);
        imagecopy($newimg,$xim,0,0,0,0,$this->layer['photo'][1],$this->layer['photo'][2]);
        imagedestroy($xim);

        // rotate the photo
        $color = imagecolorallocate($newimg, 255,0,255);
        imageSaveAlpha($newimg, true);
        $newimg = imagerotate($newimg, 5, -1);

        // place on main image
        imagecopy($this->img,$newimg,15,20,0,0,imagesx($newimg),imagesy($newimg));
        imagedestroy($newimg);
    }

    /**
     * Calculates the aspect ratio to use when resizing into bounding box
     */
    function getResizeRatio($w,$h,$maxwidth,$maxheight=0){
        if(!$maxheight) $maxheight = $maxwidth;

        $ratio = 1;
        if($w >= $h){
            if($w >= $maxwidth){
                $ratio = $maxwidth/$w;
            }elseif($h > $maxheight){
                $ratio = $maxheight/$h;
            }
        }else{
            if($h >= $maxheight){
                $ratio = $maxheight/$h;
            }elseif($w > $maxwidth){
                $ratio = $maxwidth/$w;
            }
        }
        return $ratio;
    }

    /**
     * Fetch the google map and apply Sepia filter
     */
    private function placemap(){
        $url = 'http://maps.google.com/staticmap?center='.
               $this->ll.'&zoom=14&size='.($this->width - 100).'x'.($this->height - 100).
               '&key='.$this->apikey.'&sensor=false&format=png32';

        // fetch map from google and create GD image
        $data = $this->fetchurl($url);
        $mapim = imagecreatefromstring($data);
        if(!$mapim) die("failed to create image from map");

        // apply filter
        $this->imagetosepia($mapim);

        imagecopy($this->img,$mapim,50,50,0,0,($this->width-100),($this->height-100));
        imagedestroy($mapim);

        $xim = imagecreatefrompng($this->layer['x'][0]);
        if(!$xim) die('Failed to load layer x');
        imageSaveAlpha($xim, true);
        imagecopy($this->img,$xim,
                  $this->width/2 - $this->layer['x'][1]/2,
                  $this->height/2 - $this->layer['x'][2]/2,
                  0,0,
                  $this->layer['x'][1],
                  $this->layer['x'][2]);
        imagedestroy($xim);
    }

    /**
     * Write the text at the bottom
     */
    private function placetext(){
        $color = imagecolorallocate($this->img, 119,0,0);
        imagettftext($this->img,
                     30,
                     3,
                     25,
                     $this->height-25,
                     $color,
                     $this->font,
                     $this->text);

        // reverse geocoder
        $loc = $this->fetchurl('http://maps.google.com/maps/geo?q='.$this->ll.'&oe=iso8859-1'.
                               '&output=csv&sensor=false&key='.$this->apikey);
        $loc = explode("\n",$loc);
        $loc = explode('"',$loc[0]);
        $loc = explode(',',$loc[1]);
        if(isset($loc[0]))
            imagettftext($this->img,
                         18,
                         20,
                         $this->width/2 - 70,
                         $this->height/2 + 70,
                         $color,
                         $this->font,
                         $loc[0]);

        if(isset($loc[1]))
            imagettftext($this->img,
                         18,
                         20,
                         $this->width/2 - 60,
                         $this->height/2 + 105,
                         $color,
                         $this->font,
                         $loc[1]);
    }

    /**
     * Apply an old parchment overlay
     */
    private function alphaoverlay(){
        $xim = imagecreatefrompng($this->layer['alpha'][0]);
        if(!$xim) die('Failed to load layer alpha');
        imageSaveAlpha($xim, true);
        imagecopy($this->img,$xim,0,0,0,0,$this->layer['alpha'][1],$this->layer['alpha'][2]);
        imagedestroy($xim);
    }

    /**
     * Apply a border mask
     */
    private function maskoverlay(){
        $xim = imagecreatefrompng($this->layer['mask'][0]);
        if(!$xim) die('Failed to load layer mask');

        imagealphablending($this->img, false);
        for ($x = 0; $x < $this->layer['mask'][1]; $x++) {
            for ($y = 0; $y < $this->layer['mask'][2]; $y++) {

                // current color
                $rgb = imagecolorat($this->img, $x, $y);
                $r   = ($rgb >> 16) & 0xFF;
                $g   = ($rgb >> 8) & 0xFF;
                $b   = $rgb & 0xFF;

                // red channel of mask
                $rgb   = imagecolorat($xim, $x, $y);
                $alpha = ($rgb >> 16) & 0xFF;
                $alpha = floor($alpha/2);

                $color = imagecolorallocatealpha($this->img,$r,$g,$b,$alpha);
                imagesetpixel($this->img,$x,$y,$color);
            }
        }
        imageSaveAlpha($this->img, true);
    }


    /**
     * Add the compass image in the upper right corner
     */
    private function compassoverlay(){
        $xim = imagecreatefrompng($this->layer['compass'][0]);
        if(!$xim) die('Failed to load layer compass');
        imageSaveAlpha($xim, true);
        imagecopy($this->img,$xim,
                  $this->width-$this->layer['compass'][1]-5,
                  5,0,0,
                  $this->layer['compass'][1],$this->layer['compass'][2]);
        imagedestroy($xim);
    }

    /**
     * Downloads the data from the given URL.
     *
     * needs url_fopen = On
     *
     * Replace with Curl or other methods
     */
    private function fetchurl($url){
        $data = file_get_contents($url);
        if(!$data) die("failed to fetch $url");
        return $data;
    }

    /**
     * A sepia filter
     *
     * @author <derek at idreams dot co dot uk>
     * @link   http://www.php.net/manual/en/function.imagecolorsforindex.php#79511
     */
    private function imagetosepia(&$img) {
        if (!($t = imagecolorstotal($img))) {
            $t = 256;
            imagetruecolortopalette($img, true, $t);
        }
        $total = imagecolorstotal( $img );
        for ( $i = 0; $i < $total; $i++ ) {
            $index = imagecolorsforindex( $img, $i );
            $red = ( $index["red"] * 0.393 + $index["green"] * 0.769 + $index["blue"] * 0.189 );
            $green = ( $index["red"] * 0.349 + $index["green"] * 0.686 + $index["blue"] * 0.168 );
            $blue = ( $index["red"] * 0.272 + $index["green"] * 0.534 + $index["blue"] * 0.131 );
            if ($red > 255) { $red = 255; }
            if ($green > 255) { $green = 255; }
            if ($blue > 255) { $blue = 255; }
            imagecolorset( $img, $i, $red, $green, $blue );
        }
    }




}


$md = new MapDecorate();
$md->apikey    = 'ABQIAAAAwcgTzX14Lq48uGhuAgaC-RT2yXp_ZAY8_ufC3CFXhHIE1NvwkxSoniRWQPYZHIWuWPbij8hFqvrEuw';
$md->grayscale = true;

$md->create('17.006478,-61.767368',
            'The X marks the spot! L.J. Silver',
            'http://farm4.static.flickr.com/3413/3215906731_d306912b1f.jpg');
